using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.Misc;
using Infragistics.Win.UltraWinListView;
using Infragistics.Win.Misc.UltraWinNavigationBar;
using System.Data.OleDb;

namespace DataBinding
{
    public partial class Form1 : Form
    {
        DataSet dataSet = null;

        public Form1()
        {
            InitializeComponent();

            this.InitializeListView( this.listView );

            this.navigationBar.InitializeLocations += new InitializeLocationsHandler(this.navigationBar_InitializeLocations);
            this.navigationBar.SelectedLocationChanged += new SelectedLocationChangedHandler(this.navigationBar_SelectedLocationChanged);
            
            this.PopulateNavigationBar( this.navigationBar );
        }

        private void InitializeListView( UltraListView listView )
        {
            listView.View = UltraListViewStyle.Details;

            listView.MainColumn.Text = "Order #";
            listView.MainColumn.DataType = typeof(string);

            UltraListViewSubItemColumn unitPriceColumn = listView.SubItemColumns.Add( "UnitPrice" );
            unitPriceColumn.DataType = typeof(decimal);
            unitPriceColumn.Text = "Unit Price (USD)";
            unitPriceColumn.Format = "$ #.00";
            unitPriceColumn.SubItemAppearance.TextHAlign = HAlign.Right;

            UltraListViewSubItemColumn quantityColumn = listView.SubItemColumns.Add( "Quantity" );
            quantityColumn.DataType = typeof(int);
            quantityColumn.SubItemAppearance.TextHAlign = HAlign.Right;

            UltraListViewSubItemColumn discountColumn = listView.SubItemColumns.Add( "Discount" );
            discountColumn.DataType = typeof(int);
            discountColumn.Text = "Discount (%)";
            discountColumn.Format = "# %";
            discountColumn.NullText = "None";
            discountColumn.SubItemAppearance.TextHAlign = HAlign.Right;

            UltraListViewSubItemColumn totalColumn = listView.SubItemColumns.Add( "Total" );
            totalColumn.DataType = typeof(decimal);
            totalColumn.Text = "Total w/Discount";
            totalColumn.Format = "$ #.00";
            totalColumn.SubItemAppearance.TextHAlign = HAlign.Right;
        }

        private void PopulateNavigationBar( UltraNavigationBar navigationBar )
        {
            //  Create a DataSet
            this.dataSet = new DataSet();

            //  FIll the DataSet with the Customer records
            this.customersDataAdapter.Fill( this.dataSet );

            //  Configure the root location
            navigationBar.RootLocation.Text = "Customers";

            //  Add an UltraNavigationBarLocation for each customer
            DataTable customersTable = this.dataSet.Tables["Customers"];
            foreach ( DataRow customerRow in customersTable.Rows )
            {
                //  Add an UltraNavigationBarLocation for this customer
                UltraNavigationBarLocation customerLocation = navigationBar.RootLocation.Locations.Add( null, customerRow["CompanyName"].ToString() );

                //  Set the Tag property to reference the underlying DataRow
                customerLocation.Tag = customerRow;
            }
        }

        private void navigationBar_InitializeLocations(object sender, InitializeLocationsEventArgs e)
        {
            UltraNavigationBar navigationBar = sender as UltraNavigationBar;

            //  If the 'Orders' table has not yet been created, create it
            //  by filling the DataSet.
            if ( this.dataSet.Tables.Contains("Orders") == false )
            {
                this.ordersDataAdapter.Fill( this.dataSet );

                //  Create a DataRelation for the customers and their respective orders.
                DataRelation customers_orders =
                    new DataRelation( "Customers->Orders",
                                      this.dataSet.Tables["Customers"].Columns["CustomerID"],  
                                      this.dataSet.Tables["Orders"].Columns["CustomerID"],
                                      false );

                this.dataSet.Relations.Add( customers_orders );
            }

            if ( e.ParentLocation.Level == 1 )
            {
                DataRow customerRow = e.ParentLocation.Tag as DataRow;
                DataRow[] orders = customerRow.GetChildRows( this.dataSet.Relations["Customers->Orders"] );
                foreach ( DataRow orderRow in orders )
                {
                    UltraNavigationBarLocation location = e.ParentLocation.Locations.Add( null, orderRow["OrderID"].ToString() );
                    location.Tag = orderRow;
                }
            }

        }

        private void navigationBar_SelectedLocationChanged(object sender, SelectedLocationChangedEventArgs e)
        {
            UltraNavigationBarLocation location = e.SelectedLocation;
            this.listView.Items.Clear();

            if ( location.Level == 2 )
            {
                //  If the 'Order Details' table has not yet been created, create it
                //  by filling the DataSet.
                if ( this.dataSet.Tables.Contains("Order Details") == false )
                {
                    this.orderDetailsDataAdapter.Fill( this.dataSet );

                    //  Create a DataRelation for the customers and their respective orders.
                    DataRelation orders_details =
                        new DataRelation( "Orders->Details",
                                          this.dataSet.Tables["Orders"].Columns["OrderID"],  
                                          this.dataSet.Tables["Order Details"].Columns["OrderID"],
                                          false );

                    this.dataSet.Relations.Add( orders_details );
                }

                //  Populate the ListView with the order details
                List<UltraListViewItem> items = new List<UltraListViewItem>();
                DataRow orderRow = location.Tag as DataRow;
                DataRow[] details = orderRow.GetChildRows( this.dataSet.Relations["Orders->Details"] );
                foreach ( DataRow detailsRow in details )
                {
                    Type type = detailsRow["Discount"].GetType();
                    List<object> subItemValues = new List<object>(4);
                    decimal unitPrice = (decimal)detailsRow["UnitPrice"];
                    short quantity = (short)detailsRow["Quantity"];
                    float discount = (float)detailsRow["Discount"];
                    decimal total = unitPrice * quantity;
                    total = total - (decimal)((float)total * discount);

                    subItemValues.Add( unitPrice );
                    subItemValues.Add( quantity );

                    if ( discount == 0f )
                        subItemValues.Add( null );
                    else
                        subItemValues.Add( discount );

                    subItemValues.Add( total );

                    UltraListViewItem item = new UltraListViewItem( detailsRow["OrderID"], subItemValues.ToArray() );
                    items.Add( item );
                }


                this.listView.Items.AddRange( items.ToArray() );

            }
        }


    }
}